@@ -50,13 +50,9 @@ in this Software without prior written authorization from The Open Group.
 #ifndef X_WCHAR
 #include <stddef.h>
 #else
-#ifdef __UNIXOS2__
-#include <stdlib.h>
-#else
 /* replace this with #include or typedef appropriate for your system */
 typedef unsigned long wchar_t;
 #endif
-#endif
 
 
 extern int
