@@ -510,13 +510,21 @@ _XimDefaultResName(
     Xic			  ic = (Xic)parm;
     Xim			  im = (Xim)ic->core.im;
     char		**out;
-
+    char		 *string;
+    
     if(im->core.res_name == (char *)NULL) {
 	return True;
     }
 
+   string=strdup(im->core.res_name);
+   if ( string == NULL)
+	return False;
+    
     out = (char **)((char *)top + info->offset);
-    *out = im->core.res_name;
+
+    Xfree(*out);  /* free old im->core.res_name */
+    *out =string;
+    
     return True;
 }
 
@@ -529,14 +537,22 @@ _XimDefaultResClass(
 {
     Xic			  ic = (Xic)parm;
     Xim			  im = (Xim)ic->core.im;
+    char          *string;
     char		**out;
 
     if(im->core.res_class == (char *)NULL) {
 	return True;
     }
 
+    string=strdup(im->core.res_class);
+    if (string == NULL)
+      return False;
+    
     out = (char **)((char *)top + info->offset);
-    *out = im->core.res_class;
+  
+    Xfree(*out);  /* free old im->core.res_class */
+    *out = string;
+    
     return True;
 }
 
@@ -820,9 +836,8 @@ _XimEncodeString(
     }
 
     out = (char **)((char *)top + info->offset);
-    if(*out) {
-	Xfree(*out);
-    }
+    
+    Xfree(*out);
     *out = string;
     return True;
 }
@@ -1927,7 +1942,7 @@ static const XimICMode ic_mode[] = {
 		(XIM_MODE_STS_DEFAULT | XIM_MODE_STS_SET | XIM_MODE_STS_GET),
 		0},
     {OFFSET_XNAREA,
-		0,
+		(XIM_MODE_PRE_SET | XIM_MODE_PRE_GET),
 		(XIM_MODE_PRE_DEFAULT | XIM_MODE_PRE_SET | XIM_MODE_PRE_GET),
 		(XIM_MODE_PRE_DEFAULT | XIM_MODE_PRE_SET | XIM_MODE_PRE_GET),
 		0,
