@@ -30,11 +30,7 @@ from The Open Group.
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#if USE_XCB
 #include "Xxcbint.h"
-#else /* !USE_XCB */
-#include <X11/Xtrans/Xtrans.h>
-#endif /* USE_XCB */
 #include "Xlib.h"
 #include "Xlibint.h"
 #include "Xintconn.h"
@@ -51,6 +47,7 @@ XCloseDisplay (
 {
 	register _XExtension *ext;
 	register int i;
+	xcb_connection_t *connection;
 
 	if (!(dpy->flags & XlibDisplayClosing))
 	{
@@ -69,14 +66,11 @@ XCloseDisplay (
 		    (*ext->close_display)(dpy, &ext->codes);
 	    }
 	    /* if the closes generated more protocol, sync them up */
-	    if (dpy->request != dpy->last_request_read)
+	    if (X_DPY_GET_REQUEST(dpy) != X_DPY_GET_LAST_REQUEST_READ(dpy))
 		XSync(dpy, 1);
 	}
-#if USE_XCB
-	xcb_disconnect(dpy->xcb->connection);
-#else /* !USE_XCB */
-	_XDisconnectDisplay(dpy->trans_conn);
-#endif /* USE_XCB */
+	connection = dpy->xcb->connection;
 	_XFreeDisplayStructure (dpy);
+	xcb_disconnect(connection);
 	return 0;
 }
