@@ -98,19 +98,10 @@ xthread_t (*_Xthread_self_fn)(void) = NULL;
 #define ECHECK(err) (WSAGetLastError() == err)
 #define ESET(val) WSASetLastError(val)
 #else
-#ifdef __UNIXOS2__
-#define ECHECK(err) (errno == err)
-#define ESET(val)
-#else
 #define ECHECK(err) (errno == err)
 #define ESET(val) errno = val
 #endif
-#endif
 
-#ifdef __UNIXOS2__
-#include <limits.h>
-#define MAX_PATH _POSIX_PATH_MAX
-#endif
 
 /*
  * The following routines are internal routines used by Xlib for protocol
@@ -1706,7 +1697,7 @@ _XData32(
  *       and so, you may be better off using gethostname (if it exists).
  */
 
-#if (defined(_POSIX_SOURCE) && !defined(AIXV3) && !defined(__QNX__)) || defined(hpux) || defined(SVR4)
+#if defined(_POSIX_SOURCE) || defined(SVR4)
 #define NEED_UTSNAME
 #include <sys/utsname.h>
 #else
@@ -1802,8 +1793,11 @@ void *_XGetRequest(Display *dpy, CARD8 type, size_t len)
     dpy->last_req = dpy->bufptr;
 
     req = (xReq*)dpy->bufptr;
-    req->reqType = type;
-    req->length = len / 4;
+    *req = (xReq) {
+        .reqType = type,
+        .data = 0,
+        .length = len / 4
+    };
     dpy->bufptr += len;
     X_DPY_REQUEST_INCREMENT(dpy);
     return req;
@@ -1857,10 +1851,6 @@ static int AccessFile (path, pathbuf, len_pathbuf, pathret)
 
     /* try the places set in the environment */
     drive = getenv ("_XBASEDRIVE");
-#ifdef __UNIXOS2__
-    if (!drive)
-	drive = getenv ("X11ROOT");
-#endif
     if (!drive)
 	drive = "C:";
     len = strlen (drive) + strlen (path);
@@ -1873,7 +1863,6 @@ static int AccessFile (path, pathbuf, len_pathbuf, pathret)
 	return 1;
     }
 
-#ifndef __UNIXOS2__
     /* one last place to look */
     drive = getenv ("HOMEDRIVE");
     if (drive) {
@@ -1908,7 +1897,6 @@ static int AccessFile (path, pathbuf, len_pathbuf, pathret)
 	    }
 	}
     }
-#endif
     return 0;
 }
 
