@@ -24,27 +24,31 @@
 #include <sys/select.h>
 #endif
 
-#define xcb_fail_assert(_message, _var) { \
+#define xcb_fail_assert(_message, _var) do { \
 	unsigned int _var = 1; \
 	fprintf(stderr, "[xcb] Aborting, sorry about that.\n"); \
 	assert(!_var); \
-}
+} while (0)
 
-#define throw_thread_fail_assert(_message, _var) { \
+#define throw_thread_fail_assert(_message, _var) do { \
 	fprintf(stderr, "[xcb] " _message "\n"); \
-	fprintf(stderr, "[xcb] Most likely this is a multi-threaded client " \
-	                "and XInitThreads has not been called\n"); \
+        if (_Xglobal_lock) { \
+            fprintf(stderr, "[xcb] You called XInitThreads, this is not your fault\n"); \
+        } else { \
+            fprintf(stderr, "[xcb] Most likely this is a multi-threaded client " \
+                            "and XInitThreads has not been called\n"); \
+        } \
 	xcb_fail_assert(_message, _var); \
-}
+} while (0)
 
 /* XXX: It would probably be most useful if we stored the last-processed
  *      request, so we could find the offender from the message. */
-#define throw_extlib_fail_assert(_message, _var) { \
+#define throw_extlib_fail_assert(_message, _var) do { \
 	fprintf(stderr, "[xcb] " _message "\n"); \
 	fprintf(stderr, "[xcb] This is most likely caused by a broken X " \
 	                "extension library\n"); \
 	xcb_fail_assert(_message, _var); \
-}
+} while (0)
 
 static void return_socket(void *closure)
 {
@@ -214,7 +218,12 @@ static int handle_error(Display *dpy, xError *err, Bool in_XReply)
 static void widen(uint64_t *wide, unsigned int narrow)
 {
 	uint64_t new = (*wide & ~((uint64_t)0xFFFFFFFFUL)) | narrow;
-	*wide = new + (((uint64_t)(new < *wide)) << 32);
+	/* If just copying the upper dword of *wide makes the number
+	 * go down by more than 2^31, then it means that the lower
+	 * dword has wrapped (or we have skipped 2^31 requests, which
+	 * is hopefully improbable), so we add a carry. */
+	uint64_t wraps = new + (1UL << 31) < *wide;
+	*wide = new + (wraps << 32);
 }
 
 /* Thread-safety rules:
