@@ -32,7 +32,6 @@
  *
  */
 
-#define NEED_REPLIES
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -153,7 +152,7 @@ PassToServer:
     GetReq(AllocNamedColor, req);
 
     req->cmap = cmap;
-    nbytes = req->nbytes = strlen(colorname);
+    nbytes = req->nbytes = (CARD16) strlen(colorname);
     req->length += (nbytes + 3) >> 2; /* round up to mult of 4 */
 
     _XSend(dpy, colorname, nbytes);
